/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file genheap.h
 *
 * @type subsystem
 *
 * @description
 *
 * The genheap subsystem allows applications to maintain multiple heaps
 * of malloc-style data, ostensively for keeping multiple files with
 * their own memory areas.
 */

#ifndef __rov_genheap_h
#define __rov_genheap_h


#ifdef __cplusplus
extern "C"
{
#endif

/**
 * Initialise a memory heap.
 * @param heap_ptr the pointer to the heap.
 * @param heap_size the size of the heap.
 */
void genheap_init(void *heap_ptr,
		  int heap_size);

/**
 * Allocate memory in a heap.
 * @param heap_ptr the pointer to the heap.
 * @param heap_size the size of the heap.
 * @param size the size to allocate.
 * @return the pointer in the heap, or NULL if failed.
 */
void *genheap_malloc(void *heap_ptr,
		     int heap_size,
		     int size);

/**
 * Free memory in a heap.
 * @param heap_ptr the pointer to the heap.
 * @param heap_size the size of the heap.
 * @param ptr the pointer to the memory to free.
 */
void genheap_free(void *heap_ptr,
		  int heap_size,
		  void * ptr);

/**
 * Get the free space in the heap.
 * @param heap_ptr the pointer to the heap.
 * @param heap_size the size of the heap.
 * @return the free space in the heap.
 */
int genheap_freespace(void *heap_ptr,
		      int heap_size);

/**
 * Resize a heap.
 * @param heap_ptr the pointer to the heap.
 * @param old_size the old size.
 * @param new_size the new size.
 * @note This does not perform any form of memory allocation on the heap, so
 * this must have been done beforehand.
 * @note If the new size is smaller than the allocated size, then it is
 * truncated.
 */
void genheap_resize(void *heap_ptr,
		    int old_size,
		    int new_size);

/**
 * Walk through the heap.
 * @param heap_ptr the pointer to the heap.
 * @param heap_size the heap size.
 * @param chunk the pointer to the current heap chunk (or NULL to start).
 * @return the next heap chunk (or NULL for no more).
 * @see genheap_size()
 * @see genheap_type()
 */
void * genheap_walk(void *heap_ptr,
		    int heap_size,
		    void *chunk);

/**
 * Get the size of the heap chunk
 * @param chunk the pointer to the heap chunk.
 * @return the size of the memory allocated.
 * @see genheap_walk()
 * @see genheap_type()
 */
int genheap_size(void *chunk);

/**
 * Get the type of the heap chunk
 * @param chunk the pointer to the heap chunk.
 * @return the type ('f' is a free chunk, 'a' is an allocated chunk).
 * @see genheap_walk()
 * @see genheap_size()
 */
char genheap_type(void *chunk);


#ifdef __cplusplus
}
#endif

#endif
